#!/usr/bin/env python

import numpy
import os, sys
import matplotlib.pyplot as pyplot
import matplotlib.image as image

fig = pyplot.figure()
ax = fig.add_subplot(111, axisbg='black')


_linecolor = 'white'
# change axis color
ax.spines['bottom'].set_color(_linecolor)
ax.spines['top'].set_color(_linecolor) 
ax.spines['right'].set_color(_linecolor)
ax.spines['left'].set_color(_linecolor)
# change tick color
ax.tick_params(axis='x', colors=_linecolor)
ax.tick_params(axis='y', colors=_linecolor)
# change label color
ax.yaxis.label.set_color(_linecolor)
ax.xaxis.label.set_color(_linecolor)
# change title color
ax.title.set_color(_linecolor)

fig.set_facecolor('#303030')

basename = sys.argv[1]

planets = sys.argv[2].split(",")

x_range = [float(v) for v in sys.argv[3].split(":")]
y_range = [float(v) for v in sys.argv[4].split(":")]

rel_size = float(sys.argv[5])

xlabel = sys.argv[6]
ylabel = sys.argv[7]
xcols = [int(d)-1 for d in sys.argv[8].split(",")]
ycols = [int(d)-1 for d in sys.argv[9].split(",")]

#print xcols, ycols

title = sys.argv[10]

ax.set_xlabel(xlabel)
ax.set_ylabel(ylabel)
ax.set_title(title)

planet_images = {
#     'titan': 'planets/104653main_titan_cassini_portal.jpg',
# #    'earth': 'planets/bluemarblewest.jpg',
#     'earth': 'planets/earth_small.jpg',
#     'jupiter': 'planets/Jupiter_and_its_shrunken_Great_Red_Spot.jpg',
#     'neptune': 'planets/neptune_voy2.jpg',
#     'saturn': 'planets/saturn_voy2.jpg',
#     'uranus': 'planets/uranus_voy2.jpg',
    'titan': 'planets/titan.png',
    'earth': 'planets/earth.png',
    'jupiter': 'planets/jupiter.png',
    'neptune': 'planets/neptune.png',
    'saturn': 'planets/saturn.png',
    'uranus': 'planets/uranus.png',
    'venus': 'planets/venus.png',
    'mars': 'planets/mars.png',
    'mercury': 'planets/mercury.png',
    'moon': 'planets/moon.png',
#    
    'water': 'planets/oceania.png',
    'sand': 'planets/tatooine.png',
    'vegetation': 'planets/endor.png',
    'snow': 'planets/hoth.png',

    
    }

#print basename
#print planets

ax.set_xlim(x_range)
ax.set_ylim(y_range)

ax.grid(b=True,
        which='major',
        axis='both',
        c='#b0b0b0',
        zorder=-1,
    )


xedge = [x_range[0]+0.1*(x_range[1]-x_range[0]),
         x_range[1]-0.1*(x_range[1]-x_range[0])]
yedge = [y_range[0]+0.1*(y_range[1]-y_range[0]),
         y_range[1]-0.1*(y_range[1]-y_range[0])]

extreme_x = [1e9,-1e9]
extreme_y = [1e9,-1e9]

for planet in planets:

    _planet = planet
    bonus = 0.
    if (planet.find("+") > 0):
        bonus = float(planet[planet.find("+")+1:])
        _planet = planet[:planet.find("+")]

    #print "using name", _planet

    _xsize = (rel_size*(1+bonus/100.)) * (x_range[1]-x_range[0]) * 0.5
    _ysize = (rel_size*(1+bonus/100.)) * (y_range[1]-y_range[0]) * 0.5

    data_filename = basename % _planet
    #print data_filename

    if (not os.path.isfile(data_filename)):
        continue

    data = numpy.loadtxt(data_filename)
    if (data.ndim == 1):
        data = data.reshape((1, -1))

    #print "data shape:", data.shape
    if (len(xcols) == 1):
        _x = data[:,xcols[0]]
    elif (len(xcols) == 2):
        _x = data[:,xcols[0]] - data[:, xcols[1]]
    elif (len(xcols) == 3):
        dx = float(sys.argv[8].split(",")[2])
        _x = data[:,xcols[0]] - data[:, xcols[1]] + dx

    if (len(ycols) == 1):
        _y = data[:,ycols[0]]
    elif (len(ycols) == 2):
        _y = data[:,ycols[0]] - data[:, ycols[1]]
    elif (len(ycols) == 3):
        dy = float(sys.argv[8].split(",")[2])
        _y = data[:,ycols[0]] - data[:, ycols[1]] + dy

    #_x = (data[:,1]-data[:,2])
    #_y = (data[:,2]-data[:,3])
    ax.scatter(x=_x, y=_y)

    
    # Now try to find the image for this planet
    if (_planet in planet_images):
        img_file = planet_images[_planet]
#        print img_file
        if (os.path.isfile(img_file)):
            im = image.imread(img_file)
        
#            print im.shape

            for point in range(_x.shape[0]):
#                print _x[point], _y[point]

                __x = _x[point]
                __y = _y[point]

                if (__x < xedge[0] or __x > xedge[1] or
                    __y < yedge[0] or __y > yedge[1]):
                    print "point close to edge (%s): %.3f %.3f" % (planet, __x, __y)

                extreme_x[0] = __x if __x < extreme_x[0] else extreme_x[0]
                extreme_x[1] = __x if __x > extreme_x[1] else extreme_x[1]

                extreme_y[0] = __y if __y < extreme_y[0] else extreme_y[0]
                extreme_y[1] = __y if __y > extreme_y[1] else extreme_y[1]

                #fig.figimage(im, xo=0, yo=0, alpha=0.9, origin='lower')
                #print "showing image"
                ax.imshow(im, aspect='auto', 
                          extent=(__x-_xsize, __x+_xsize, __y-_ysize, __y+_ysize),
                          zorder=2,
                )
        else:
            print "file not found:",  img_file
    else:
        print "unknown planet", _planet

try:
    extrafile = sys.argv[12]
    print extrafile
    data = numpy.loadtxt(extrafile)
   
    print data.shape

    if (len(xcols) == 1):
        _x = data[:,xcols[0]]
    elif (len(xcols) == 2):
        _x = data[:,xcols[0]] - data[:, xcols[1]]
    elif (len(xcols) == 3):
        dx = float(sys.argv[8].split(",")[2])
        _x = data[:,xcols[0]] - data[:, xcols[1]] + dx

    if (len(ycols) == 1):
        _y = data[:,ycols[0]]
    elif (len(ycols) == 2):
        _y = data[:,ycols[0]] - data[:, ycols[1]]
    elif (len(ycols) == 3):
        dy = float(sys.argv[8].split(",")[2])
        _y = data[:,ycols[0]] - data[:, ycols[1]] + dy

    print "loadin done"
    ax.scatter(_x, _y, c='white', linewidth=0, alpha=0.35, s=25)
    print "done scattering"

except:
    pass

print "\nextrema: x: %.3f / %.3f     y: %.3f / %.3f\n"  %(
    extreme_x[0], extreme_x[1], extreme_y[0], extreme_y[1])

fig.tight_layout()
#fig.patch.set_facecolor('#202020')
#fig.show()
#pyplot.show()
fig.set_size_inches(8,6)
fig.savefig(sys.argv[11], dpi=400, facecolor=fig.get_facecolor(), edgecolor='none')
